/*
    (C) 1995-2001 AROS - The Amiga Research OS
    $Id: lockrecords.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc:
    Lang: English
*/
#include "dos_intern.h"
#include <proto/exec.h>

/*****************************************************************************

    NAME */
#include <dos/record.h>
#include <proto/dos.h>

	AROS_LH2(LONG, LockRecords,

/*  SYNOPSIS */
	AROS_LHA(struct RecordLock *, recArray, D1),
	AROS_LHA(ULONG              , timeout, D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 46, Dos)

/*  FUNCTION

    Lock several records at the same time. The timeout specified is applied
    to each lock to attempt. The array of RecordLock:s is terminated with
    an entry where rec_FH is equal to NULL.

    INPUTS

    recArray  --  array of records to lock
    timeout   --  maximum number of ticks to wait for a lock to be ready

    RESULT

    Success/failure indication. In case of a success, all the record locks
    are locked. In case of failure, no record locks are locked.

    NOTES

    A set of records should always be locked in the same order so as to
    reduce possiblities of deadlock.

    EXAMPLE

    BUGS

    SEE ALSO

    UnLockRecords()

    INTERNALS

    HISTORY

*****************************************************************************/
{
    AROS_LIBFUNC_INIT

    struct RecordLock *rLock;

    FAILFORTASKNP(return DOSFALSE;)

    for (rLock = recArray; rLock->rec_FH; rLock++)
    {
	if (!LockRecord(rLock->rec_FH, rLock->rec_Offset,
			rLock->rec_Length, rLock->rec_Mode, timeout))
	{
	    LONG ioerr = IoErr();

	    while (rLock-- != recArray)
	    {
		UnLockRecord(rLock->rec_FH, rLock->rec_Offset,
		             rLock->rec_Length);
	    }

	    SetIoErr(ioerr);

	    return DOSFALSE;
	}
    }

    return DOSTRUE;

    AROS_LIBFUNC_EXIT
} /* LockRecords */
